@@ -1,28 +1,20 @@
-/*  Lzlib - Compression library for lzip files
-    Copyright (C) 2009, 2010, 2011, 2012, 2013 Antonio Diaz Diaz.
+/*  Lzlib - Compression library for the lzip format
+    Copyright (C) 2009-2017 Antonio Diaz Diaz.
 
-    This library is free software: you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation, either version 3 of the License, or
-    (at your option) any later version.
+    This library is free software. Redistribution and use in source and
+    binary forms, with or without modification, are permitted provided
+    that the following conditions are met:
+
+    1. Redistributions of source code must retain the above copyright
+    notice, this list of conditions and the following disclaimer.
+
+    2. Redistributions in binary form must reproduce the above copyright
+    notice, this list of conditions and the following disclaimer in the
+    documentation and/or other materials provided with the distribution.
 
     This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with this library.  If not, see <http://www.gnu.org/licenses/>.
-
-    As a special exception, you may use this file as part of a free
-    software library without restriction.  Specifically, if other files
-    instantiate templates or use macros or inline functions from this
-    file, or you compile this file and link it with other files to
-    produce an executable, this file does not by itself cause the
-    resulting executable to be covered by the GNU General Public
-    License.  This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General
-    Public License.
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
 
 #ifndef max
@@ -56,14 +48,16 @@ static inline State St_set_short_rep( const State st )
 
 enum {
   min_dictionary_bits = 12,
-  min_dictionary_size = 1 << min_dictionary_bits,
+  min_dictionary_size = 1 << min_dictionary_bits,	/* >= modeled_distances */
   max_dictionary_bits = 29,
   max_dictionary_size = 1 << max_dictionary_bits,
   literal_context_bits = 3,
+  literal_pos_state_bits = 0,				/* not used */
   pos_state_bits = 2,
   pos_states = 1 << pos_state_bits,
   pos_state_mask = pos_states - 1,
 
+  len_states = 4,
   dis_slot_bits = 6,
   start_dis_model = 4,
   end_dis_model = 14,
@@ -81,12 +75,10 @@ enum {
 
   min_match_len = 2,					/* must be 2 */
   max_match_len = min_match_len + max_len_symbols - 1,	/* 273 */
-  min_match_len_limit = 5,
+  min_match_len_limit = 5 };
 
-  dis_states = 4 };
-
-static inline int get_dis_state( const int len )
-  { return min( len - min_match_len, dis_states - 1 ); }
+static inline int get_len_state( const int len )
+  { return min( len - min_match_len, len_states - 1 ); }
 
 static inline int get_lit_state( const uint8_t prev_byte )
   { return ( prev_byte >> ( 8 - literal_context_bits ) ); }
@@ -101,8 +93,8 @@ typedef int Bit_model;
 static inline void Bm_init( Bit_model * const probability )
   { *probability = bit_model_total / 2; }
 
-static inline void Bm_array_init( Bit_model * const p, const int size )
-  { int i = 0; while( i < size ) p[i++] = bit_model_total / 2; }
+static inline void Bm_array_init( Bit_model bm[], const int size )
+  { int i; for( i = 0; i < size; ++i ) Bm_init( &bm[i] ); }
 
 struct Len_model
   {
@@ -175,14 +167,22 @@ static inline void CRC32_update_byte( uint32_t * const crc, const uint8_t byte )
   { *crc = crc32[(*crc^byte)&0xFF] ^ ( *crc >> 8 ); }
 
 static inline void CRC32_update_buf( uint32_t * const crc,
-                                     const uint8_t * const buffer, const int size )
+                                     const uint8_t * const buffer,
+                                     const int size )
   {
   int i;
+  uint32_t c = *crc;
   for( i = 0; i < size; ++i )
-    *crc = crc32[(*crc^buffer[i])&0xFF] ^ ( *crc >> 8 );
+    c = crc32[(c^buffer[i])&0xFF] ^ ( c >> 8 );
+  *crc = c;
   }
 
 
+static inline bool isvalid_ds( const unsigned dictionary_size )
+  { return ( dictionary_size >= min_dictionary_size &&
+             dictionary_size <= max_dictionary_size ); }
+
+
 static inline int real_bits( unsigned value )
   {
   int bits = 0;
@@ -204,6 +204,14 @@ static inline void Fh_set_magic( File_header data )
 static inline bool Fh_verify_magic( const File_header data )
   { return ( memcmp( data, magic_string, 4 ) == 0 ); }
 
+/* detect truncated header */
+static inline bool Fh_verify_prefix( const File_header data, const int size )
+  {
+  int i; for( i = 0; i < size && i < 4; ++i )
+    if( data[i] != magic_string[i] ) return false;
+  return ( size > 0 );
+  }
+
 static inline uint8_t Fh_version( const File_header data )
   { return data[4]; }
 
@@ -220,28 +228,25 @@ static inline unsigned Fh_get_dictionary_size( const File_header data )
 
 static inline bool Fh_set_dictionary_size( File_header data, const unsigned sz )
   {
-  if( sz >= min_dictionary_size && sz <= max_dictionary_size )
+  if( !isvalid_ds( sz ) ) return false;
+  data[5] = real_bits( sz - 1 );
+  if( sz > min_dictionary_size )
     {
-    data[5] = real_bits( sz - 1 );
-    if( sz > min_dictionary_size )
-      {
-      const unsigned base_size = 1 << data[5];
-      const unsigned wedge = base_size / 16;
-      int i;
-      for( i = 7; i >= 1; --i )
-        if( base_size - ( i * wedge ) >= sz )
-          { data[5] |= ( i << 5 ); break; }
-      }
-    return true;
+    const unsigned base_size = 1 << data[5];
+    const unsigned fraction = base_size / 16;
+    unsigned i;
+    for( i = 7; i >= 1; --i )
+      if( base_size - ( i * fraction ) >= sz )
+        { data[5] |= ( i << 5 ); break; }
     }
-  return false;
+  return true;
   }
 
 static inline bool Fh_verify( const File_header data )
   {
-  return ( Fh_verify_magic( data ) && Fh_verify_version( data ) &&
-           Fh_get_dictionary_size( data ) >= min_dictionary_size &&
-           Fh_get_dictionary_size( data ) <= max_dictionary_size );
+  if( Fh_verify_magic( data ) && Fh_verify_version( data ) )
+    return isvalid_ds( Fh_get_dictionary_size( data ) );
+  return false;
   }
 
 
@@ -255,41 +260,29 @@ enum { Ft_size = 20 };
 static inline unsigned Ft_get_data_crc( const File_trailer data )
   {
   unsigned tmp = 0;
-  int i;
-  for( i = 3; i >= 0; --i ) { tmp <<= 8; tmp += data[i]; }
+  int i; for( i = 3; i >= 0; --i ) { tmp <<= 8; tmp += data[i]; }
   return tmp;
   }
 
 static inline void Ft_set_data_crc( File_trailer data, unsigned crc )
-  {
-  int i;
-  for( i = 0; i <= 3; ++i ) { data[i] = (uint8_t)crc; crc >>= 8; }
-  }
+  { int i; for( i = 0; i <= 3; ++i ) { data[i] = (uint8_t)crc; crc >>= 8; } }
 
 static inline unsigned long long Ft_get_data_size( const File_trailer data )
   {
   unsigned long long tmp = 0;
-  int i;
-  for( i = 11; i >= 4; --i ) { tmp <<= 8; tmp += data[i]; }
+  int i; for( i = 11; i >= 4; --i ) { tmp <<= 8; tmp += data[i]; }
   return tmp;
   }
 
 static inline void Ft_set_data_size( File_trailer data, unsigned long long sz )
-  {
-  int i;
-  for( i = 4; i <= 11; ++i ) { data[i] = (uint8_t)sz; sz >>= 8; }
-  }
+  { int i; for( i = 4; i <= 11; ++i ) { data[i] = (uint8_t)sz; sz >>= 8; } }
 
 static inline unsigned long long Ft_get_member_size( const File_trailer data )
   {
   unsigned long long tmp = 0;
-  int i;
-  for( i = 19; i >= 12; --i ) { tmp <<= 8; tmp += data[i]; }
+  int i; for( i = 19; i >= 12; --i ) { tmp <<= 8; tmp += data[i]; }
   return tmp;
   }
 
 static inline void Ft_set_member_size( File_trailer data, unsigned long long sz )
-  {
-  int i;
-  for( i = 12; i <= 19; ++i ) { data[i] = (uint8_t)sz; sz >>= 8; }
-  }
+  { int i; for( i = 12; i <= 19; ++i ) { data[i] = (uint8_t)sz; sz >>= 8; } }
